package com.ruoyi.web.controller.system;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ArrayUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ASSERT;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.utils.sql.DBUtil;
import com.ruoyi.framework.web.service.SysPermissionService;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.common.AppEnv;
import com.ruoyi.system.domain.SysPost;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysPostService;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.web.core.config.ResponseTypeMap;
import com.ruoyi.web.core.config.ResponseTypeMapGroup;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 用户信息
 *
 * @author ruoyi
 */
@Api(tags = {"移动端用户管理"})
@RestController
@RequestMapping("/system/user")
public class SysUserController extends BaseController
{
    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private ISysPostService postService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private SysUserRoleMapper         sysUserRoleMapper;

    @Autowired
    private TokenService tokenService;

    /**
     * 获取用户列表
     */
    @PreAuthorize("@ss.hasPermi('system:user:list')")
    @GetMapping("/list")
    public TableDataInfo list(SysUser user)
    {
        startPage();
        List<SysUser> list = userService.selectUserList(user);
        return getDataTable(list);
    }

    @Log(title = "用户管理", businessType = BusinessType.EXPORT)
    @PreAuthorize("@ss.hasPermi('system:user:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, SysUser user)
    {
        List<SysUser> list = userService.selectUserList(user);
        ExcelUtil<SysUser> util = new ExcelUtil<SysUser>(SysUser.class);
        util.exportExcel(response, list, "用户数据");
    }

    @Log(title = "用户管理", businessType = BusinessType.IMPORT)
    @PreAuthorize("@ss.hasPermi('system:user:import')")
    @PostMapping("/importData")
    public AjaxResult importData(MultipartFile file, boolean updateSupport) throws Exception
    {
        ExcelUtil<SysUser> util = new ExcelUtil<SysUser>(SysUser.class);
        List<SysUser> userList = util.importExcel(file.getInputStream());
        String operName = getUsername();
        String message = userService.importUser(userList, updateSupport, operName);
        return success(message);
    }

    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<SysUser> util = new ExcelUtil<SysUser>(SysUser.class);
        util.importTemplateExcel(response, "用户数据");
    }

    /**
     * 根据用户编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:user:query')")
    @GetMapping(value = { "/", "/{userId}" })
    public AjaxResult getInfo(@PathVariable(value = "userId", required = false) Long userId)
    {
        userService.checkUserDataScope(userId);
        AjaxResult ajax = AjaxResult.success();
        List<SysRole> roles = roleService.selectRoleAll();
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect(Collectors.toList()));
        ajax.put("posts", postService.selectPostAll());
        if (StringUtils.isNotNull(userId))
        {
            SysUser sysUser = userService.selectUserById(userId);
            ajax.put(AjaxResult.DATA_TAG, sysUser);
            ajax.put("postIds", postService.selectPostListByUserId(userId));
            ajax.put("roleIds", sysUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
        }
        return ajax;
    }

    /**
     * 新增用户
     */
    @PreAuthorize("@ss.hasPermi('system:user:add')")
    @Log(title = "用户管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysUser user)
    {
        if(StringUtils.isEmpty(user.getUserName()))
            user.setUserName(user.getPhonenumber());
        if (!userService.checkUserNameUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !userService.checkPhoneUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        else if (StringUtils.isNotEmpty(user.getEmail()) && !userService.checkEmailUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setCreateBy(getUsername());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        return toAjax(userService.insertUser(user));
    }



    /**
     * 修改用户
     */
    @PreAuthorize("@ss.hasPermi('system:user:edit')")
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysUser user)
    {
        userService.checkUserAllowed(user);
        userService.checkUserDataScope(user.getUserId());
        if (!userService.checkUserNameUnique(user))
        {
            return error("修改用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !userService.checkPhoneUnique(user))
        {
            return error("修改用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        else if (StringUtils.isNotEmpty(user.getEmail()) && !userService.checkEmailUnique(user))
        {
            return error("修改用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setUpdateBy(getUsername());
        return toAjax(userService.updateUser(user));
    }

    /**
     * 删除用户
     */
    @PreAuthorize("@ss.hasPermi('system:user:remove')")
    @Log(title = "用户管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{userIds}")
    public AjaxResult remove(@PathVariable Long[] userIds)
    {
        if (ArrayUtils.contains(userIds, getUserId()))
        {
            return error("当前用户不能删除");
        }
        return toAjax(userService.deleteUserByIds(userIds));
    }

    /**
     * 重置密码
     */
    @PreAuthorize("@ss.hasPermi('system:user:resetPwd')")
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping("/resetPwd")
    public AjaxResult resetPwd(@RequestBody SysUser user)
    {
        userService.checkUserAllowed(user);
        userService.checkUserDataScope(user.getUserId());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        user.setUpdateBy(getUsername());
        return toAjax(userService.resetPwd(user));
    }

    /**
     * 状态修改
     */
    @PreAuthorize("@ss.hasPermi('system:user:edit')")
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping("/changeStatus")
    public AjaxResult changeStatus(@RequestBody SysUser user)
    {
        userService.checkUserAllowed(user);
        userService.checkUserDataScope(user.getUserId());
        user.setUpdateBy(getUsername());
        return toAjax(userService.updateUserStatus(user));
    }

    /**
     * 根据用户编号获取授权角色
     */
    @PreAuthorize("@ss.hasPermi('system:user:query')")
    @GetMapping("/authRole/{userId}")
    public AjaxResult authRole(@PathVariable("userId") Long userId)
    {
        AjaxResult ajax = AjaxResult.success();
        SysUser user = userService.selectUserById(userId);
        List<SysRole> roles = roleService.selectRolesByUserId(userId);
        ajax.put("user", user);
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect(Collectors.toList()));
        return ajax;
    }

    /**
     * 用户授权角色
     */
    @PreAuthorize("@ss.hasPermi('system:user:edit')")
    @Log(title = "用户管理", businessType = BusinessType.GRANT)
    @PutMapping("/authRole")
    public AjaxResult insertAuthRole(Long userId, Long[] roleIds)
    {
        userService.checkUserDataScope(userId);
        userService.insertUserAuth(userId, roleIds);
        return success();
    }

    /**
     * 获取部门树列表
     */
    //@PreAuthorize("@ss.hasPermi('system:user:list')")
    @GetMapping("/deptTree")
    public AjaxResult deptTree(SysDept dept)
    {
        return success(deptService.selectDeptTreeList(dept));
    }



    /**
     * 用户认证
     */
    @ApiOperation("农户认证(姓名name、证件号idCard)")
    @Log(title = "用户认证", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/auth")
    public AjaxResult auth() {
        return AjaxResult.success();
    }

    @ApiOperation("用户切换角色")
    @GetMapping(value = "/changRole/{roleId}")
    public AjaxResult changRole(@PathVariable("roleId") Long roleId)
    {
        ASSERT.TRUE(ArrayUtil.contains(AppEnv.APP_ROLE_IDS, roleId), "该角色不允许登录App端");
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        SysUser userCureent = loginUser.getUser();

        // 检查角色是否可切换
        List<SysRole> sysRoles = roleService.selectUserRoleList(userCureent.getUserId());
        ASSERT.EXP(sysRoles.stream().anyMatch((x) -> roleId.longValue() == x.getRoleId()), "您无权切换此角色");

        userCureent.setCurrentRole(roleId);
        userCureent.updateRoleIds();
        userService.userChangeRole(userCureent);

        // 刷新token
        tokenService.setLoginUser(loginUser);

        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", userCureent);
        return ajax;
    }

    @ApiOperation("用户切换角色(角色字符串)")
    @GetMapping(value = "/changRoleByKey/{roleKey}")
    public AjaxResult changRoleByKey(@PathVariable("roleKey") String roleKey)
    {
        ASSERT.TRUE(ArrayUtil.contains(AppEnv.APP_ROLE_KEYS, roleKey), "该角色不允许登录App端");
        SysRole role = roleService.selectRoleByKey(roleKey);
        ASSERT.NONNULL(role, "角色被禁用");
        return changRole(role.getRoleId());
    }

    /**
     * 当前用户详细信息(实时)
     */
    @ApiOperation("当前用户详细信息(实时)")
    @ResponseTypeMapGroup({
            @ResponseTypeMap(name = "data", desc = "用户信息", type = SysUser.class),
            @ResponseTypeMap(name = "posts", desc = "职务列表", type = List.class, genericType = SysPost.class),
            @ResponseTypeMap(name = "roleIds", desc = "角色ID列表", type = List.class, genericType = Long.class),
            @ResponseTypeMap(name = "postIds", desc = "职务ID列表", type = List.class, genericType = Long.class),
            @ResponseTypeMap(name = "roleKeys", desc = "角色名称列表", type = List.class, genericType = String.class),
    })
    @GetMapping("/myprofile")
    public AjaxResult myprofile()
    {
        Long userId = getUserId();
        SysUser user = userService.selectUserById(userId);

        List<SysRole> roles = user.getRoles();
        roles.removeIf((x) -> !ArrayUtil.contains(AppEnv.APP_ROLE_KEYS, x.getRoleKey()));
        if(CollectionUtil.isNotEmpty(roles))
        {
            roles = roles.stream().filter((x) -> ArrayUtil.contains(AppEnv.APP_ROLE_KEYS, x.getRoleKey())).collect(Collectors.toList());
            if(CollectionUtil.isNotEmpty(roles))
            {
                SysRole role;
                boolean roleChanged = false;
                if(!ArrayUtil.contains(AppEnv.APP_ROLE_IDS, user.getCurrentRole()))
                {
                    role = roles.get(0);
                    roleChanged = true;
                }
                else
                {
                    Optional<SysRole> first = roles.stream().filter((x) -> x.getRoleId().equals(user.getCurrentRole())).findFirst();
                    if(first.isPresent())
                        role = first.get();
                    else
                    {
                        role = roles.get(0);
                        roleChanged = true;
                    }
                }
                if(roleChanged)
                {
                    user.setCurrentRole(role.getRoleId());
                    userService.updateUserProfile(user);
                    LoginUser loginUser = getLoginUser();
                    loginUser.getUser().setCurrentRole(role.getRoleId());
                    tokenService.setLoginUser(loginUser);
                }
            }
        }

        AjaxResult ajax = AjaxResult.success();
        ajax.put(AjaxResult.DATA_TAG, user);
        ajax.put("posts", postService.selectPostAll());
        ajax.put("postIds", postService.selectPostListByUserId(userId));
        ajax.put("roleIds", user.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
        ajax.put("roleKeys", user.getRoles().stream().map(SysRole::getRoleKey).collect(Collectors.toList()));
        user.setCurrentRoleKey(user.getRoles().stream().filter((x) -> x.getRoleId().equals(user.getCurrentRole())).map(SysRole::getRoleKey).findFirst().orElse(null));
        return ajax;
    }

    /**
     * 取消用户认证
     */
    @PreAuthorize("@ss.hasPermi('admin:user:revokeAuth')")
    @Log(title = "取消用户认证", businessType = BusinessType.UPDATE)
    @PostMapping("/revokeAuth")
    public AjaxResult revokeAuth(@RequestParam Long userId)
    {
        ASSERT.EXP(getLoginUser().getUser().isAdmin(), "只有超级管理员才可操作");
        SysUser user = userService.selectUserById(userId);
        if(!user.auth())
            return AjaxResult.success("用户未认证");

        DBUtil.transaction(() -> {
            sysUserRoleMapper.deleteSpecialUserRole(userId, 6L);
            user.setAuthStatus("0");
            userService.updateUserStatus(user);
        });

        return AjaxResult.success();
    }

    /**
     * 获取用户列表
     */
    @PreAuthorize("@ss.hasPermi('system:user:list')")
    @GetMapping("/detailList")
    public TableDataInfo detailList(SysUser user)
    {
        mustPage();
        List<SysUser> list = userService.selectUserDetailList(user);
        return getDataTable(list);
    }
}
