package com.ruoyi.file.mapper;

import java.util.List;
import com.ruoyi.file.domain.Project;

/**
 * 项目Mapper接口
 * 
 * @author zhao
 * @date 2023-10-19
 */
public interface ProjectMapper 
{
    /**
     * 查询项目
     * 
     * @param id 项目主键
     * @return 项目
     */
    public Project selectProjectById(Long id);

    /**
     * 查询项目列表
     * 
     * @param project 项目
     * @return 项目集合
     */
    public List<Project> selectProjectList(Project project);

    /**
     * 新增项目
     * 
     * @param project 项目
     * @return 结果
     */
    public int insertProject(Project project);

    /**
     * 批量新增项目
     *
     * @param list 项目
     * @return 结果
     */
    public int insertProjectBatch(List<Project> list);

    /**
     * 修改项目
     * 
     * @param project 项目
     * @return 结果
     */
    public int updateProject(Project project);

    /**
     * 批量修改 项目
     *
     * @param list 项目
     * @return 结果
     */
    public int updateProjectBatch(List<Project> list);

    /**
     * 删除项目
     * 
     * @param id 项目主键
     * @return 结果
     */
    public int deleteProjectById(Long id);

    /**
     * 批量删除项目
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteProjectByIds(Long[] ids);

    /**
     * 条件单条查询项目
     *
     * @param project 项目
     * @return 项目条目
     */
    public Project selectProject(Project project);

    /**
     * 条件查询项目数量
     *
     * @param project 项目
     * @return 项目数量
     */
    public Long selectProjectCount(Project project);

    /**
     * 条件查询项目是否存在
     *
     * @param project 项目
     * @return 项目是否存在
     */
    public int selectProjectExists(Project project);
}
